@@ -29,6 +29,8 @@
 #' @param ylab a `string`. (Default is `""`). Y label.
 #' @param xlab a `string`. (Default is `"Profile Index"`). X label.
 #' @param ... further arguments to be passed to [plot()]. See [par()].
+#' @param xmin an `int`. (Default is `NULL`). Set the minimum value of x axis.
+#' @param xmax an `int`. (Default is `NULL`). Set the maximum value of x axis.
 #'
 #' @return None
 #' @keywords hplot
@@ -37,9 +39,20 @@
 #' @examples
 #' plot_arcs(pairs = matrix(c(5, 10, 1, 10, 20, 5), ncol = 2, byrow = TRUE))
 plot_arcs <- function(pairs, alpha = NULL, quality = 30, lwd = 15, col = c("blue", "orange"),
-                      main = "Arc Plot", ylab = "", xlab = "Profile Index", ...) {
-  xmin <- min(pairs)
-  xmax <- max(pairs)
+                      main = "Arc Plot", ylab = "", xlab = "Profile Index", xmin = NULL, xmax = NULL, ...) {
+  if (length(pairs) == 0) {
+    warning("No arc to plot.")
+    return(NULL)
+  }
+
+  if (is.null(xmin)) {
+    xmin <- min(pairs)
+  }
+
+  if (is.null(xmax)) {
+    xmax <- max(pairs)
+  }
+
   max_arc <- max(abs(pairs[, 2] - pairs[, 1]))
   ymax <- (max_arc / 2 + (lwd * lwd) / 8)
   z_seq <- seq(0, base::pi, length.out = quality)
@@ -141,24 +154,41 @@ plot.ArcCount <- function(x, data, type = c("data", "matrix"), exclusion_zone =
     data_lab <- ylab
     data_main <- "Data"
   } else {
-    plot_data <- x$mp
+    plot_data <- c(x$mp, rep(NA, x$w - 1))
     data_lab <- "distance"
     data_main <- "Matrix Profile"
   }
 
   cac <- x$cac # keep cac intact
-
   cac_size <- length(cac)
+  profile_index <- x$pi
+
+  if (cac_size < nrow(profile_index)) {
+    warning("cac_size < profile_index")
+    cac_offset <- nrow(profile_index) - cac_size
+    plot_data <- as.matrix(utils::tail(plot_data, cac_size))
+    profile_index <- as.matrix(utils::tail(profile_index, cac_size) - cac_offset)
+  }
+
+  xnum <- seq_len(nrow(x$mp) + min(x$w) - 1)
+
+  offset <- attr(x, "offset")
+  offset <- ifelse(is.null(offset), 0, offset)
+
+  if (!is.null(offset)) {
+    xnum <- xnum + offset
+  }
+
   pairs <- matrix(0, cac_size, 2)
-  pairs[, 1] <- seq_len(cac_size)
-  pairs[, 2] <- x$pi
+  pairs[, 1] <- seq_len(cac_size) + offset
+  pairs[, 2] <- profile_index + offset
 
   if (threshold < min(cac)) {
-    stop(paste0("Error: `threshold` is too small for this Arc Count. Min: ", round(min(cac), 2), ", Max: ", round(max(cac), 2)))
+    stop(paste0("`threshold` is too small for this Arc Count. Min: ", round(min(cac), 2), ", Max: ", round(max(cac), 2)))
   }
 
   # remove excess of arcs
-  if (floor(x$w * exclusion_zone) < length(x$mp) / 3) {
+  if (floor(x$w * exclusion_zone) < nrow(x$mp) / 3) {
     exclusion_zone <- floor(x$w * exclusion_zone)
   }
   if (floor(x$w * edge_limit) < length(x$mp) / 3) {
@@ -168,22 +198,22 @@ plot.ArcCount <- function(x, data, type = c("data", "matrix"), exclusion_zone =
   cac[(cac_size - edge_limit + 1):cac_size] <- Inf
 
   ind <- which(cac <= threshold)
-  pairs <- pairs[ind, ]
+  pairs <- pairs[ind, , drop = FALSE]
 
   pairdiff <- pairs[, 1] - pairs[, 2]
   ind <- which(abs(pairdiff) > exclusion_zone)
-  pairs <- pairs[ind, ]
+  pairs <- pairs[ind, , drop = FALSE]
 
-  xmin <- min(pairs)
-  xmax <- max(pairs)
+  xmin <- min(xnum)
+  xmax <- max(xnum)
   xlim <- c(xmin, xmax)
 
   graphics::layout(matrix(c(1, 2, 3), ncol = 1, byrow = TRUE))
   graphics::par(oma = c(1, 1, 3, 0), cex.lab = 1.5)
-  plot_arcs(pairs, xlab = xlab, ...)
+  plot_arcs(pairs, xlab = xlab, xmin = xmin, xmax = xmax, ...)
   graphics::mtext(text = main, font = 2, cex = 1.5, outer = TRUE)
-  graphics::plot(x$cac, main = "Arc count", type = "l", xlab = xlab, ylab = "normalized count", xlim = xlim, ...)
-  graphics::plot(plot_data, main = data_main, type = "l", xlab = xlab, ylab = data_lab, xlim = xlim, ...)
+  graphics::plot(xnum, c(cac, rep(NA, x$w - 1)), main = "Arc count", type = "l", xlab = xlab, ylab = "normalized count", xlim = xlim, ...)
+  graphics::plot(xnum, plot_data, main = data_main, type = "l", xlab = xlab, ylab = data_lab, xlim = xlim, ...)
 
   graphics::par(def_par)
 }
@@ -192,27 +222,40 @@ plot.ArcCount <- function(x, data, type = c("data", "matrix"), exclusion_zone =
 #' @keywords hplot
 #' @name plot
 #'
-plot.Valmod <- function(x, ylab = "distance", xlab = "index", main = "Valmod Matrix Profile", ...) {
+plot.Valmod <- function(x, ylab = "distance", xlab = "index", main = "Valmod Matrix Profile", data = FALSE, ...) {
   def_par <- graphics::par(no.readonly = TRUE)
   allmatrix <- FALSE
+  num_charts <- 1
 
-  if (!is.null(x$lmp) && !all(x$lpi == -1)) {
-    allmatrix <- TRUE
+  xnum <- seq_len(nrow(x$mp) + min(x$w) - 1)
+
+  if (!is.null(attr(x, "offset"))) {
+    xnum <- xnum + attr(x, "offset")
   }
 
-  if (allmatrix == TRUE) {
-    graphics::layout(matrix(c(1, 2, 3), ncol = 1, byrow = TRUE))
+  if (data) {
+    num_charts <- num_charts + 1
+  }
+
+  if (num_charts > 1) {
+    graphics::layout(matrix(seq_len(num_charts), ncol = 1, byrow = TRUE))
   }
   graphics::par(
     mar = c(4.1, 4.1, 2.1, 2.1),
     oma = c(1, 1, 3, 0), cex.lab = 1.5
   )
-  graphics::plot(x$mp, type = "l", main = paste0("Matrix Profile (w = ", min(x$w), "-", max(x$w), "; ez = ", x$ez, ")"), ylab = ylab, xlab = xlab, ...)
+
+  if (data) {
+    graphics::plot(xnum, x$data[[1]], type = "l", main = paste0("Data"), ylab = ylab, xlab = xlab, ...)
+    graphics::mtext(text = main, font = 2, cex = 1.5, outer = TRUE)
+  }
+
+  graphics::plot(xnum, c(x$mp, rep(NA, min(x$w) - 1)), type = "l", main = paste0("Matrix Profile (w = ", min(x$w), "-", max(x$w), "; ez = ", x$ez, ")"), ylab = ylab, xlab = xlab, ...)
   graphics::mtext(text = main, font = 2, cex = 1.5, outer = TRUE)
 
   if (allmatrix == TRUE) {
-    graphics::plot(x$rmp, type = "l", main = "Right Matrix Profile", ylab = ylab, xlab = xlab, ...)
-    graphics::plot(x$lmp, type = "l", main = "Left Matrix Profile", ylab = ylab, xlab = xlab, ...)
+    graphics::plot(xnum, c(x$rmp, rep(NA, min(x$w) - 1)), type = "l", main = "Right Matrix Profile", ylab = ylab, xlab = xlab, ...)
+    graphics::plot(xnum, c(x$lmp, rep(NA, min(x$w) - 1)), type = "l", main = "Left Matrix Profile", ylab = ylab, xlab = xlab, ...)
   }
 
   graphics::par(def_par)
@@ -222,27 +265,45 @@ plot.Valmod <- function(x, ylab = "distance", xlab = "index", main = "Valmod Mat
 #' @keywords hplot
 #' @name plot
 #'
-plot.MatrixProfile <- function(x, ylab = "distance", xlab = "index", main = "Unidimensional Matrix Profile", ...) {
+plot.MatrixProfile <- function(x, ylab = "distance", xlab = "index", main = "Unidimensional Matrix Profile", data = FALSE, ...) {
   def_par <- graphics::par(no.readonly = TRUE)
   allmatrix <- FALSE
+  num_charts <- 1
 
-  if (!is.null(x$lmp) && !all(x$lpi == -1)) {
-    allmatrix <- TRUE
+  xnum <- seq_len(nrow(x$mp) + x$w - 1)
+
+  if (!is.null(attr(x, "offset"))) {
+    xnum <- xnum + attr(x, "offset")
   }
 
-  if (allmatrix == TRUE) {
-    graphics::layout(matrix(c(1, 2, 3), ncol = 1, byrow = TRUE))
+  if (!is.null(attr(x, "join")) && !attr(x, "join")) {
+    allmatrix <- TRUE
+    num_charts <- 3
+  }
+
+  if (data) {
+    num_charts <- num_charts + 1
+  }
+
+  if (num_charts > 1) {
+    graphics::layout(matrix(seq_len(num_charts), ncol = 1, byrow = TRUE))
   }
   graphics::par(
     mar = c(4.1, 4.1, 2.1, 2.1),
     oma = c(1, 1, 3, 0), cex.lab = 1.5
   )
-  graphics::plot(x$mp, type = "l", main = paste0("Matrix Profile (w = ", x$w, "; ez = ", x$ez, ")"), ylab = ylab, xlab = xlab, ...)
+
+  if (data) {
+    graphics::plot(xnum, x$data[[1]], type = "l", main = paste0("Data"), ylab = "", xlab = xlab, ...)
+    graphics::mtext(text = main, font = 2, cex = 1.5, outer = TRUE)
+  }
+
+  graphics::plot(xnum, c(x$mp, rep(NA, x$w - 1)), type = "l", main = paste0("Matrix Profile (w = ", x$w, "; ez = ", x$ez, ")"), ylab = ylab, xlab = xlab, ...)
   graphics::mtext(text = main, font = 2, cex = 1.5, outer = TRUE)
 
   if (allmatrix == TRUE) {
-    graphics::plot(x$rmp, type = "l", main = "Right Matrix Profile", ylab = ylab, xlab = xlab, ...)
-    graphics::plot(x$lmp, type = "l", main = "Left Matrix Profile", ylab = ylab, xlab = xlab, ...)
+    graphics::plot(xnum, c(x$rmp, rep(NA, x$w - 1)), type = "l", main = "Right Matrix Profile", ylab = ylab, xlab = xlab, ...)
+    graphics::plot(xnum, c(x$lmp, rep(NA, x$w - 1)), type = "l", main = "Left Matrix Profile", ylab = ylab, xlab = xlab, ...)
   }
 
   graphics::par(def_par)
@@ -255,30 +316,41 @@ plot.MatrixProfile <- function(x, ylab = "distance", xlab = "index", main = "Uni
 plot.MultiMatrixProfile <- function(x, ylab = "distance", xlab = "index", main = "Multidimensional Matrix Profile", ...) {
   def_par <- graphics::par(no.readonly = TRUE)
   allmatrix <- FALSE
+  num_charts <- 1
+  mask <- !is.na(x$mp[1, ])
+  x$mp <- x$mp[, mask, drop = FALSE]
   n_dim <- ncol(x$mp)
 
-  if (!is.null(x$lmp) && !all(x$lpi == -1)) {
+  xnum <- seq_len(nrow(x$mp) + x$w - 1)
+
+  if (!is.null(attr(x, "offset"))) {
+    xnum <- xnum + attr(x, "offset")
+  }
+
+  if (!is.null(attr(x, "join")) && !attr(x, "join")) {
     allmatrix <- TRUE
+    num_charts <- 3
   }
 
   if (allmatrix == TRUE) {
-    graphics::layout(matrix(seq_len(3 * n_dim), ncol = 3, byrow = TRUE))
+    graphics::layout(matrix(seq_len(num_charts * n_dim), ncol = n_dim, byrow = TRUE))
   }
+
   graphics::par(
     mar = c(4.1, 4.1, 2.1, 2.1),
     oma = c(1, 1, 3, 0), cex.lab = 1.5
   )
   for (i in seq_len(n_dim)) {
-    graphics::plot(x$mp[, i], type = "l", main = paste0("Matrix Profile (w = ", x$w, "; ez = ", x$ez, ")"), ylab = ylab, xlab = xlab, ...)
+    graphics::plot(xnum, c(x$mp[, i], rep(NA, min(x$w) - 1)), type = "l", main = paste0("Matrix Profile (w = ", x$w, "; ez = ", x$ez, ")"), ylab = ylab, xlab = xlab, ...)
   }
   graphics::mtext(text = main, font = 2, cex = 1.5, outer = TRUE)
 
   if (allmatrix == TRUE) {
     for (i in seq_len(n_dim)) {
-      graphics::plot(x$rmp[, i], type = "l", main = "Right Matrix Profile", ylab = ylab, xlab = xlab, ...)
+      graphics::plot(xnum, c(x$rmp[, i], rep(NA, min(x$w) - 1)), type = "l", main = "Right Matrix Profile", ylab = ylab, xlab = xlab, ...)
     }
     for (i in seq_len(n_dim)) {
-      graphics::plot(x$lmp[, i], type = "l", main = "Left Matrix Profile", ylab = ylab, xlab = xlab, ...)
+      graphics::plot(xnum, c(x$lmp[, i], rep(NA, min(x$w) - 1)), type = "l", main = "Left Matrix Profile", ylab = ylab, xlab = xlab, ...)
     }
   }
 
@@ -289,36 +361,58 @@ plot.MultiMatrixProfile <- function(x, ylab = "distance", xlab = "index", main =
 #' @keywords hplot
 #' @name plot
 #'
-plot.SimpleMatrixProfile <- function(x, ylab = "distance", xlab = "index", main = "SiMPle Matrix Profile", ...) {
+plot.SimpleMatrixProfile <- function(x, ylab = "distance", xlab = "index", main = "SiMPle Matrix Profile", data = FALSE, ...) {
   def_par <- graphics::par(no.readonly = TRUE)
-  allmatrix <- FALSE
-  n_dim <- ncol(x$mp)
+  num_charts <- 1
 
-  if (!is.null(x$lmp) && !all(x$lpi == -1)) {
-    allmatrix <- TRUE
+  xnum <- seq_len(nrow(x$mp) + x$w - 1)
+
+  if (!is.null(attr(x, "offset"))) {
+    xnum <- xnum + attr(x, "offset")
   }
 
-  if (allmatrix == TRUE) {
-    graphics::layout(matrix(seq_len(3 * n_dim), ncol = 3, byrow = TRUE))
+  if (data) {
+    num_charts <- num_charts + length(x$data)
   }
+
+  if (num_charts > 1) {
+    graphics::layout(matrix(seq_len(num_charts), ncol = 1, byrow = TRUE))
+  }
+
   graphics::par(
     mar = c(4.1, 4.1, 2.1, 2.1),
     oma = c(1, 1, 3, 0), cex.lab = 1.5
   )
-  for (i in seq_len(n_dim)) {
-    graphics::plot(x$mp[, i], type = "l", main = paste0("Matrix Profile (w = ", x$w, "; ez = ", x$ez, ")"), ylab = ylab, xlab = xlab, ...)
+
+  if (data) {
+    n_dim <- ncol(x$data[[1]])
+
+    graphics::plot(xnum, x$data[[1]][, 1], type = "l", main = paste0("Data"), ylab = "", xlab = xlab, ...)
+    graphics::mtext(text = main, font = 2, cex = 1.5, outer = TRUE)
+
+    if (n_dim > 1) {
+      for (i in 2:n_dim) {
+        graphics::lines(xnum, x$data[[1]][, i], main = paste0("Data"), ylab = "", xlab = xlab, col = i, ...)
+      }
+    }
+
+    if (length(x$data) > 1) {
+      n_dim <- ncol(x$data[[2]])
+
+      graphics::plot(xnum, x$data[[2]][, 1], type = "l", main = paste0("Query"), ylab = "", xlab = xlab, ...)
+      graphics::mtext(text = main, font = 2, cex = 1.5, outer = TRUE)
+
+      if (n_dim > 1) {
+        for (i in 2:n_dim) {
+          graphics::lines(xnum, x$data[[2]][, i], main = paste0("Data"), ylab = "", xlab = xlab, col = i, ...)
+        }
+      }
+    }
   }
+
+  graphics::plot(xnum, c(x$mp, rep(NA, min(x$w) - 1)), type = "l", main = paste0("Matrix Profile (w = ", x$w, "; ez = ", x$ez, ")"), ylab = ylab, xlab = xlab, ...)
+
   graphics::mtext(text = main, font = 2, cex = 1.5, outer = TRUE)
-
-  if (allmatrix == TRUE) {
-    for (i in seq_len(n_dim)) {
-      graphics::plot(x$rmp[, i], type = "l", main = "Right Matrix Profile", ylab = ylab, xlab = xlab, ...)
-    }
-    for (i in seq_len(n_dim)) {
-      graphics::plot(x$lmp[, i], type = "l", main = "Left Matrix Profile", ylab = ylab, xlab = xlab, ...)
-    }
-  }
-
   graphics::par(def_par)
 }
 
@@ -344,7 +438,7 @@ plot.Fluss <- function(x, data, type = c("data", "matrix"),
     data_lab <- ylab
     data_main <- "Data"
   } else {
-    plot_data <- x$mp
+    plot_data <- c(x$mp, rep(NA, min(x$w) - 1))
     data_lab <- "distance"
     data_main <- "Matrix Profile"
   }
@@ -354,30 +448,111 @@ plot.Fluss <- function(x, data, type = c("data", "matrix"),
   fluss_size <- length(fluss_idx) + 1
   pairs <- matrix(0, fluss_size, 2)
 
+  offset <- attr(x, "offset")
+  offset <- ifelse(is.null(offset), 0, offset)
+
   for (i in seq_len(fluss_size)) {
     if (i == 1) {
-      pairs[i, 1] <- 0
+      pairs[i, 1] <- offset
     } else {
-      pairs[i, 1] <- fluss_idx[i - 1]
+      pairs[i, 1] <- fluss_idx[i - 1] + offset
     }
 
     if (i == fluss_size) {
-      pairs[i, 2] <- length(x$mp)
+      pairs[i, 2] <- nrow(x$mp) + offset
     } else {
-      pairs[i, 2] <- fluss_idx[i]
+      pairs[i, 2] <- fluss_idx[i] + offset
     }
   }
 
-  xmin <- min(pairs)
-  xmax <- max(pairs)
+  xnum <- seq_len(nrow(x$mp) + min(x$w) - 1) + offset
+  xmin <- min(xnum)
+  xmax <- max(xnum)
   xlim <- c(xmin, xmax)
 
   graphics::layout(matrix(c(1, 2, 3), ncol = 1, byrow = TRUE))
   graphics::par(oma = c(1, 1, 3, 0), cex.lab = 1.5)
-  plot_arcs(pairs, xlab = xlab, ...)
+  plot_arcs(pairs, xlab = xlab, xmin = xmin, xmax = xmax, ...)
   graphics::mtext(text = main, font = 2, cex = 1.5, outer = TRUE)
-  graphics::plot(plot_data, main = data_main, type = "l", xlab = xlab, ylab = data_lab, xlim = xlim, ...)
-  graphics::plot(x$cac, main = "Arc count", type = "l", xlab = xlab, ylab = "normalized count", xlim = xlim, ...)
+  graphics::plot(xnum, plot_data, main = data_main, type = "l", xlab = xlab, ylab = data_lab, xlim = xlim, ...)
+  graphics::plot(xnum, c(x$cac, rep(NA, min(x$w) - 1)), main = "Arc count", type = "l", xlab = xlab, ylab = "normalized count", xlim = xlim, ylim = c(0, 1), ...)
+
+  graphics::par(def_par)
+}
+
+#' @export
+#' @keywords hplot
+#' @name plot
+#'
+plot.Floss <- function(x, data, type = c("data", "matrix"),
+                       main = "Fast Low-cost Online Semantic Segmentation", xlab = "index",
+                       ylab = "", ...) {
+  def_par <- graphics::par(no.readonly = TRUE)
+
+  if (missing(data) && !is.null(x$data)) {
+    data <- x$data[[1]]
+  } else {
+    is.null(data) # check data presence before plotting anything
+  }
+
+  type <- match.arg(type)
+
+  if (type == "data") {
+    plot_data <- data
+    data_lab <- ylab
+    data_main <- "Data"
+  } else {
+    plot_data <- c(x$mp, rep(NA, min(x$w) - 1))
+    data_lab <- "distance"
+    data_main <- "Matrix Profile"
+  }
+
+  offset <- attr(x, "offset")
+  offset <- ifelse(is.null(offset), 0, offset)
+
+  floss_idx <- sort(x$floss)
+  floss_idx <- floss_idx - offset
+
+  floss_size <- length(floss_idx) + 1
+  pairs <- matrix(0, floss_size, 2)
+
+  for (i in seq_len(floss_size)) {
+    if (i == 1) {
+      pairs[i, 1] <- 0 # offset
+    } else {
+      pairs[i, 1] <- floss_idx[i - 1] + offset
+    }
+
+    if (i == floss_size) {
+      pairs[i, 2] <- nrow(x$mp) + offset
+    } else {
+      pairs[i, 2] <- floss_idx[i] + offset
+    }
+  }
+
+  xnum <- seq_len(nrow(x$mp) + min(x$w) - 1) + offset
+  xmin <- min(xnum)
+  xmax <- max(xnum)
+  xlim <- c(xmin, xmax)
+
+  cac_fin_len <- length(x$cac_final)
+  mp_len <- nrow(x$mp)
+  new_data <- attr(x, "new_data")
+
+  if (cac_fin_len == floor((mp_len * vars()$kmode + new_data / 2))) {
+    cac <- x$cac_final
+  } else {
+    cac <- utils::tail(x$cac_final, -offset)
+  }
+  cac_size <- length(cac)
+  cac <- c(cac, rep(NA, nrow(x$mp) + min(x$w) - 1 - cac_size))
+
+  graphics::layout(matrix(c(1, 2, 3), ncol = 1, byrow = TRUE))
+  graphics::par(oma = c(1, 1, 3, 0), cex.lab = 1.5)
+  plot_arcs(pairs, xlab = xlab, xmin = xmin, xmax = xmax, ...)
+  graphics::mtext(text = main, font = 2, cex = 1.5, outer = TRUE)
+  graphics::plot(xnum, plot_data, main = data_main, type = "l", xlab = xlab, ylab = data_lab, xlim = xlim, ...)
+  graphics::plot(xnum, cac, main = "Arc count", type = "l", xlab = xlab, ylab = "normalized count", xlim = xlim, ylim = c(0, 1), ...)
 
   graphics::par(def_par)
 }
@@ -401,44 +576,47 @@ plot.Chain <- function(x, data, type = c("data", "matrix"), main = "Chain Discov
     plot_data <- data
     plot_subtitle <- "Data"
   } else {
-    plot_data <- x$mp
+    plot_data <- c(x$mp, rep(NA, min(x$w) - 1))
     ylab <- "distance"
     plot_subtitle <- paste0("Matrix Profile (w = ", x$w, "; ez = ", x$ez, ")")
   }
 
   chain_size <- length(x$chain$best)
-  pairs <- matrix(Inf, chain_size - 1, 2)
-  matrix_profile_size <- nrow(x$mp)
+  pairs <- matrix(0, chain_size - 1, 2)
+
+  offset <- attr(x, "offset")
+  offset <- ifelse(is.null(offset), 0, offset)
 
   for (i in seq_len(chain_size - 1)) {
-    pairs[i, 1] <- x$chain$best[i]
-    pairs[i, 2] <- x$chain$best[i + 1]
+    pairs[i, 1] <- x$chain$best[i] + offset
+    pairs[i, 2] <- x$chain$best[i + 1] + offset
   }
 
-  xmin <- min(pairs)
-  xmax <- max(pairs)
+  xnum <- seq_len(nrow(x$mp) + min(x$w) - 1) + offset
+  xmin <- min(xnum)
+  xmax <- max(xnum)
   xlim <- c(xmin, xmax)
 
   # plot matrix profile
   graphics::layout(matrix(c(1, 2, 3), ncol = 1, byrow = TRUE))
   graphics::par(oma = c(1, 1, 3, 0), cex.lab = 1.5)
-  plot_arcs(pairs, xlab = xlab, ...)
+  plot_arcs(pairs, xlab = xlab, xmin = xmin, xmax = xmax, ...)
   graphics::mtext(text = main, font = 2, cex = 1.5, outer = TRUE)
-  graphics::plot(plot_data,
+  graphics::plot(xnum, plot_data,
     type = "l", main = plot_subtitle,
     xlim = xlim, xlab = xlab, ylab = ylab, ...
   )
-  graphics::abline(v = x$chain$best, col = 1:chain_size, lwd = 2)
+  graphics::abline(v = x$chain$best + offset, col = 1:chain_size, lwd = 2)
 
   # blank plot
-  motif <- znorm(data[x$chain$best[1]:min((x$chain$best[1] + x$w - 1), matrix_profile_size)])
+  motif <- znorm(data[x$chain$best[1]:min((x$chain$best[1] + x$w - 1), nrow(data))])
   graphics::plot(motif,
     type = "l", main = "Motifs", xlab = "length", ylab = "normalized data",
     xlim = c(0, length(motif)), ylim = c(min(motif) - chain_size / 2, max(motif)), ...
   )
 
   for (i in 2:chain_size) {
-    motif <- znorm(data[x$chain$best[i]:min((x$chain$best[i] + x$w - 1), matrix_profile_size)])
+    motif <- znorm(data[x$chain$best[i]:min((x$chain$best[i] + x$w - 1), nrow(data))])
 
     graphics::lines(motif - i / 2, col = i, ...)
   }
@@ -483,14 +661,23 @@ plot.Discord <- function(x, data, type = c("data", "matrix"), ncol = 3, main = "
     byrow = TRUE
   ))
   # plot matrix profile
+  xnum <- seq_len(nrow(x$mp) + min(x$w) - 1)
+
+  offset <- attr(x, "offset")
+  offset <- ifelse(is.null(offset), 0, offset)
+
+  if (!is.null(offset)) {
+    xnum <- xnum + offset
+  }
+
   graphics::par(oma = c(1, 1, 3, 0), cex.lab = 1.5)
-  graphics::plot(plot_data, type = "l", main = plot_subtitle, xlab = xlab, ylab = ylab)
+  graphics::plot(xnum, plot_data, type = "l", main = plot_subtitle, xlab = xlab, ylab = ylab)
   graphics::mtext(text = main, font = 2, cex = 1.5, outer = TRUE)
-  graphics::abline(v = discords, col = seq_len(n_discords), lwd = 3)
-  graphics::abline(v = unlist(neighbors), col = rep(seq_len(n_discords), sapply(neighbors, length)), lwd = 1, lty = 2)
+  graphics::abline(v = unlist(discords) + offset, col = seq_len(n_discords), lwd = 3)
+  graphics::abline(v = unlist(neighbors) + offset, col = rep(seq_len(n_discords), sapply(neighbors, length)), lwd = 1, lty = 2)
   # plot discords
   for (i in 1:n_discords) {
-    discord1 <- znorm(data[discords[i]:min((discords[i] + x$w - 1), matrix_profile_size)])
+    discord1 <- znorm(data[discords[[i]]:min((discords[[i]] + x$w - 1), matrix_profile_size)])
 
     # blank plot
     graphics::plot(0.5, 0.5,
@@ -516,6 +703,14 @@ plot.Discord <- function(x, data, type = c("data", "matrix"), ncol = 3, main = "
 plot.Motif <- function(x, data, type = c("data", "matrix"), ncol = 3, main = "MOTIF Discover", xlab = "index", ylab = "", ...) {
   def_par <- graphics::par(no.readonly = TRUE)
 
+  motifs <- x$motif$motif_idx
+  n_motifs <- length(x$motif$motif_idx)
+
+  if (n_motifs == 0) {
+    graphics::par(def_par)
+    stop("No Motifs found to plot.")
+  }
+
   if ("Valmod" %in% class(x)) {
     valmod <- TRUE
 
@@ -538,7 +733,7 @@ plot.Motif <- function(x, data, type = c("data", "matrix"), ncol = 3, main = "MO
     plot_data <- data
     plot_subtitle <- "Data"
   } else {
-    plot_data <- x$mp
+    plot_data <- c(x$mp, rep(NA, min(x$w) - 1))
     ylab <- "distance"
     if (valmod) {
       plot_subtitle <- paste0("Matrix Profile (w = ", min(x$w), "-", max(x$w), "; ez = ", x$ez, ")")
@@ -547,11 +742,8 @@ plot.Motif <- function(x, data, type = c("data", "matrix"), ncol = 3, main = "MO
     }
   }
 
-  motifs <- x$motif$motif_idx
-  n_motifs <- length(x$motif$motif_idx)
   neighbors <- x$motif$motif_neighbor
   windows <- unlist(x$motif$motif_window)
-  matrix_profile_size <- nrow(x$mp)
 
   # layout: matrix profile on top, motifs below.
   graphics::layout(matrix(
@@ -561,26 +753,34 @@ plot.Motif <- function(x, data, type = c("data", "matrix"), ncol = 3, main = "MO
     byrow = TRUE
   ))
   # plot matrix profile
+  xnum <- seq_len(nrow(x$mp) + min(x$w) - 1)
+
+  offset <- attr(x, "offset")
+  offset <- ifelse(is.null(offset), 0, offset)
+
+  if (!is.null(offset)) {
+    xnum <- xnum + offset
+  }
   graphics::par(oma = c(1, 1, 3, 0), cex.lab = 1.5)
-  graphics::plot(plot_data, type = "l", main = plot_subtitle, xlab = xlab, ylab = ylab)
+  graphics::plot(xnum, plot_data, type = "l", main = plot_subtitle, xlab = xlab, ylab = ylab)
   graphics::mtext(text = main, font = 2, cex = 1.5, outer = TRUE)
-  graphics::abline(v = unlist(motifs), col = rep(1:n_motifs, each = 2), lwd = 3)
-  graphics::abline(v = unlist(neighbors), col = rep(1:n_motifs, sapply(neighbors, length)), lwd = 1, lty = 2)
+  graphics::abline(v = unlist(motifs) + offset, col = rep(1:n_motifs, each = 2), lwd = c(3, 1))
+  graphics::abline(v = unlist(neighbors) + offset, col = rep(1:n_motifs, sapply(neighbors, length)), lwd = 1, lty = 2)
 
   # plot motifs
   if (valmod) {
     for (i in 1:n_motifs) {
-      motif1 <- znorm(data[motifs[[i]][1]:min((motifs[[i]][1] + windows[i] - 1), matrix_profile_size)])
-      motif2 <- znorm(data[motifs[[i]][2]:min((motifs[[i]][2] + windows[i] - 1), matrix_profile_size)])
+      motif1 <- znorm(data[motifs[[i]][1]:min((motifs[[i]][1] + windows[i] - 1), nrow(data))])
+      motif2 <- znorm(data[motifs[[i]][2]:min((motifs[[i]][2] + windows[i] - 1), nrow(data))])
 
       # blank plot
       graphics::plot(0.5, 0.5,
-        type = "n", main = paste("Motif", i), sub = paste("w = ", windows[i]), xlab = "length", ylab = "normalized data",
+        type = "n", main = paste0("Motif ", i, " (w = ", windows[i], ")"), xlab = "length", ylab = "normalized data",
         xlim = c(0, length(motif1)), ylim = c(min(motif1), max(motif1))
       )
 
       for (j in seq_len(length(neighbors[[i]]))) {
-        neigh <- znorm(data[neighbors[[i]][j]:min((neighbors[[i]][j] + windows[i] - 1), matrix_profile_size)])
+        neigh <- znorm(data[neighbors[[i]][j]:min((neighbors[[i]][j] + windows[i] - 1), nrow(data))])
         graphics::lines(neigh, col = "gray70", lty = 2)
       }
 
@@ -589,8 +789,8 @@ plot.Motif <- function(x, data, type = c("data", "matrix"), ncol = 3, main = "MO
     }
   } else {
     for (i in 1:n_motifs) {
-      motif1 <- znorm(data[motifs[[i]][1]:min((motifs[[i]][1] + x$w - 1), matrix_profile_size)])
-      motif2 <- znorm(data[motifs[[i]][2]:min((motifs[[i]][2] + x$w - 1), matrix_profile_size)])
+      motif1 <- znorm(data[motifs[[i]][1]:min((motifs[[i]][1] + x$w - 1), nrow(data))])
+      motif2 <- znorm(data[motifs[[i]][2]:min((motifs[[i]][2] + x$w - 1), nrow(data))])
 
       # blank plot
       graphics::plot(0.5, 0.5,
@@ -599,7 +799,7 @@ plot.Motif <- function(x, data, type = c("data", "matrix"), ncol = 3, main = "MO
       )
 
       for (j in seq_len(length(neighbors[[i]]))) {
-        neigh <- znorm(data[neighbors[[i]][j]:min((neighbors[[i]][j] + x$w - 1), matrix_profile_size)])
+        neigh <- znorm(data[neighbors[[i]][j]:min((neighbors[[i]][j] + x$w - 1), nrow(data))])
         graphics::lines(neigh, col = "gray70", lty = 2)
       }
 
@@ -618,6 +818,14 @@ plot.Motif <- function(x, data, type = c("data", "matrix"), ncol = 3, main = "MO
 plot.MultiMotif <- function(x, data, type = c("data", "matrix"), ncol = 3, main = "Multidimensional MOTIF Discover", xlab = "index", ylab = "", ...) {
   def_par <- graphics::par(no.readonly = TRUE)
 
+  motifs <- x$motif$motif_idx
+  n_motifs <- length(x$motif$motif_idx)
+
+  if (n_motifs == 0) {
+    graphics::par(def_par)
+    stop("No Motifs found to plot.")
+  }
+
   if (missing(data) && !is.null(x$data)) {
     data <- x$data[[1]]
   } else {
@@ -630,16 +838,12 @@ plot.MultiMotif <- function(x, data, type = c("data", "matrix"), ncol = 3, main
     plot_data <- data
     plot_subtitle <- "Data"
   } else {
-    plot_data <- x$mp
+    plot_data <- apply(x$mp, 2, function(y) c(y, rep(NA, min(x$w) - 1)))
     ylab <- "distance"
-    plot_subtitle <- paste0("Matrix Profile ", i, " (w = ", x$w, "; ez = ", x$ez, ")")
   }
 
   n_dim <- x$n_dim
-  motifs <- x$motif$motif_idx
   motifs_dim <- x$motif$motif_dim
-  n_motifs <- length(x$motif$motif_idx)
-  matrix_profile_size <- nrow(x$mp)
 
   dim_idx <- list()
   for (i in seq_len(n_dim)) {
@@ -660,12 +864,25 @@ plot.MultiMotif <- function(x, data, type = c("data", "matrix"), ncol = 3, main
     byrow = TRUE
   ))
   # plot matrix profile
+  xnum <- seq_len(nrow(x$mp) + min(x$w) - 1)
+
+  offset <- attr(x, "offset")
+  offset <- ifelse(is.null(offset), 0, offset)
+
+  if (!is.null(offset)) {
+    xnum <- xnum + offset
+  }
+
   graphics::par(
     mar = c(4.1, 4.1, 2.1, 2.1),
     oma = c(1, 1, 3, 0), cex.lab = 1.5
   )
   for (i in seq_len(length(dim_idx))) {
-    graphics::plot(plot_data[, i],
+    if (type == "matrix") {
+      plot_subtitle <- paste0("Matrix Profile ", i, "d (w = ", x$w, "; ez = ", x$ez, ")")
+    }
+
+    graphics::plot(xnum, plot_data[, i],
       type = "l",
       main = plot_subtitle,
       xlab = xlab, ylab = ylab
@@ -673,7 +890,7 @@ plot.MultiMotif <- function(x, data, type = c("data", "matrix"), ncol = 3, main
 
     midx <- dim_idx[[i]]
     if (!is.null(midx)) {
-      graphics::abline(v = motifs[midx], col = midx, lwd = 2)
+      graphics::abline(v = unlist(motifs[midx]) + offset, col = rep(midx, each = 2), lwd = c(2, 1))
     }
   }
 
@@ -681,30 +898,38 @@ plot.MultiMotif <- function(x, data, type = c("data", "matrix"), ncol = 3, main
 
   # plot motifs
   for (i in 1:n_motifs) {
-    motif_data <- list()
     dim_len <- length(motifs_dim[[i]])
+
+    motif1 <- list()
+    motif2 <- list()
+
     for (j in seq_len(dim_len)) {
-      motif_data[[j]] <- znorm(data[motifs[i]:min((motifs[i] + x$w - 1), matrix_profile_size), motifs_dim[[i]][j]])
+      motif1[[j]] <- znorm(data[motifs[[i]][1]:min((motifs[[i]][1] + x$w - 1), nrow(data)), motifs_dim[[i]][j]])
+      motif2[[j]] <- znorm(data[motifs[[i]][2]:min((motifs[[i]][2] + x$w - 1), nrow(data)), motifs_dim[[i]][j]])
     }
 
+
     # blank plot
     graphics::plot(0.5, 0.5,
       type = "n", main = paste("Motif", i), xlab = "length", ylab = "normalized data",
-      xlim = c(0, length(motif_data[[1]])), ylim = c(min(unlist(motif_data)), max(unlist(motif_data)))
+      xlim = c(0, length(motif1[[1]])), ylim = c(min(unlist(motif1), unlist(motif2)), max(unlist(motif1), unlist(motif2)))
     )
 
-    if (length(motif_data) > 1) {
+    if (length(motif2) > 1) {
       for (j in (seq_len(dim_len - 1) + 1)) {
-        graphics::lines(motif_data[[j]], col = "gray30", lwd = 1)
+        graphics::lines(motif2[[j]], col = i, lwd = 1)
+        graphics::lines(motif1[[j]], col = i, lwd = 2)
       }
     }
 
-    graphics::lines(motif_data[[1]], col = i, lwd = 2)
+    graphics::lines(motif2[[1]], col = i, lwd = 1)
+    graphics::lines(motif1[[1]], col = i, lwd = 2)
   }
 
   graphics::par(def_par)
 }
 
+
 #' @export
 #' @keywords hplot
 #' @name plot
@@ -724,14 +949,23 @@ plot.Salient <- function(x, data, main = "Salient Subsections", xlab = "index",
   y_min <- min(data)
   y_max <- max(data)
 
+
+  offset <- attr(x, "offset")
+  offset <- ifelse(is.null(offset), 0, offset)
+
   mds <- salient_mds(x, data)
-  idxs <- sort(x$salient$indexes[, 1])
+  idxs <- sort(x$salient$indexes[, 1]) + offset
 
   # layout: matrix profile on top, motifs below.
   graphics::layout(matrix(c(1, 1, 1, 0, 2, 0), ncol = 3, byrow = TRUE))
   # plot matrix profile
+  xnum <- seq_len(nrow(x$mp) + min(x$w) - 1)
+
+  if (!is.null(offset)) {
+    xnum <- xnum + offset
+  }
   graphics::par(oma = c(1, 1, 3, 0), cex.lab = 1.5)
-  graphics::plot(plot_data, type = "l", main = plot_subtitle, xlab = xlab, ylab = ylab)
+  graphics::plot(xnum, plot_data, type = "l", main = plot_subtitle, xlab = xlab, ylab = ylab)
   graphics::mtext(text = main, font = 2, cex = 1.5, outer = TRUE)
 
   graphics::rect(idxs, y_min,
