@@ -4,6 +4,7 @@ if (skip_on_cran()) {
   w <- 30
   ref_data <- mp_toy_data$data[, 1]
   query_data <- mp_toy_data$data[, 1]
+  query_gap <- c(10:1, rep(NA, 10), 10:20)
   d_size <- length(ref_data)
   q_size <- length(query_data)
 
@@ -16,14 +17,30 @@ if (skip_on_cran()) {
     expect_error(tsmp:::diff2(as.matrix(1:10), matrix(10:1, ncol = 2)), "columns")
   })
 
-  pre <- mass_pre(ref_data, d_size, query_data, q_size, w)
-  res <- mass(
-    pre$data_fft, query_data[1:w], d_size, w, pre$data_mean, pre$data_sd,
-    pre$query_mean[1], pre$query_sd[1]
-  )
+  gap <- dist_profile(ref_data, query_gap, window_size = w)
+
+  test_that("Query with Gap", {
+    expect_equal(sum(round(Re(gap$distance_profile[21:541]), 2)), 38257.03)
+  })
+
+  pre <- mass_pre(ref_data, query_data, w)
+  pre$query_mean <- pre$query_mean[1]
+  pre$query_sd <- pre$query_sd[1]
+
+  pre3 <- c(pre, list(data = ref_data, k = NULL))
+
+  res <- do.call("mass_v2", (c(list(query_data[1:w]), pre)))
+  res3 <- do.call("mass_v3", (c(list(query_data[1:w]), pre3)))
+  pre3$k <- 1 # fix hash later
+
   movsd <- fast_movsd(mp_toy_data$data[, 1], 30)
   movavg <- fast_movavg(mp_toy_data$data[, 1], 30)
 
+  prew <- mass_pre_w(ref_data, query_data, w, c(rep(1, 15), rep(0.5, 15)))
+  resw <- do.call("mass_weighted", (c(list(query_data[1:w]), prew)))
+
+
+
   test_that("Fast Moving SD is ok", {
     expect_known_hash(round(movsd, 3), "ffda40fd35")
   })
@@ -33,12 +50,18 @@ if (skip_on_cran()) {
   })
 
   test_that("MASS Pre is ok", {
-    expect_known_hash(lapply(pre, round, 3), "0a8e28def6")
+    expect_equal(sum(Re(unlist(lapply(pre, round, 3)))), 1305.817)
+    expect_equal(sum(Re(unlist(lapply(pre3, round, 3)))), 1599.839)
+    expect_equal(sum(Re(unlist(lapply(prew, round, 2)))), 13147.42)
   })
 
   test_that("MASS is ok", {
     expect_equal(sum(round(Re(res$distance_profile), 2)), 30737.17)
     expect_equal(sum(round(Re(res$last_product), 2)), 5965.13)
+    expect_equal(sum(round(Re(res3$distance_profile), 2)), 30737.17)
+    expect_equal(sum(round(Re(res3$last_product), 2)), 5965.13)
+    expect_equal(sum(round(Re(resw$distance_profile), 2)), 21944.89)
+    expect_equal(sum(round(Re(resw$last_product), 2)), -1076.72)
   })
 
   pred <- rep(c(rep(1, 220), rep(0, 7830)), 27)[1:214791]
